/******************************************************
 * Inline (Google) Definitions Firefox Extension
 ******************************************************
 *
 * based in large part on the Greasemonkey script by Brent Charbonneau:
 *  Wikipedia Inline Article Viewer version 0.1.2 2005-09-03
 *  Copyright (c) 2005, Brent Charbonneau
 *  Released under the GPL license
 *  http://www.gnu.org/copyleft/gpl.html
 *
 * Also using a bit of code from "Dictionary Search" by Jaap Haitsma
 * ...this is also released under the GPL :)
 */

/* namespace - everything will be in this one global variable */
var InlineDefinitionsExtension = {};
(function() {


this.log = function (str) {
    dump("[inline-definitions] " + str + "\n");
}

this.dumpObject = function(obj) {
    dump("\n dumping object contents\n");
    for(i in obj){
        dump(i + " = " + obj[i] + "\n");
    }
    dump("\n");
}

this.init = function () {
    InlineDefinitionsExtension.log("inlineDefinitionsInit()");

    /* Register context menu event */
    document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", InlineDefinitionsExtension.onMenuShowing, false);
}

/* hide the Inline Search item when apropriate (use same logic as for web search) */
this.onMenuShowing = function (event) { 
    var selection = InlineDefinitionsExtension.getSelection();
    var selectedText = selection.join(" ");   
    var menuItem1 = document.getElementById("context_inlinedefinitions");
    var menuItem2 = document.getElementById("context_inlinedefinitions_close");
    var menuItem3 = document.getElementById("context_inlinedefinitions_back");
       
    /* Main menu item */
    if (selectedText != ""){
        menuItem1.hidden = false;        
        menuItem1.setAttribute("label", "Definitions");
    }
    else{
        menuItem1.hidden = true;
    }

    /* back & close menu items */
    if (InlineDefinitionsExtension.isInsideDefinitionsWindow(document.popupNode)) {
        menuItem1.hidden = true;

        menuItem2.hidden = false;
        menuItem2.setAttribute("label", "Inline Definitions");
        return;

        var htmlDoc = document.popupNode.ownerDocument;
        var historyNode = htmlDoc.getElementById('inlinedefinitions-history');
        var historyEntry = historyNode.lastChild;
        var backPhrase = null;
        if (historyEntry) {
            backPhrase = historyEntry.getAttribute("inlinedefinitions-search-phrase");
        }
        
        if (backPhrase == null) {
            menuItem3.hidden = true;
        } else {
            menuItem3.hidden = false;
            menuItem3.setAttribute("label", 'Back to "' + backPhrase + '"');
        }

        /* if current phrase is selected, don't give option of searching again for it.. */
        var innerWindowContentBox = htmlDoc.getElementById('inlinedefinitions-content');
        var curPhrase = innerWindowContentBox.getAttribute("inlinedefinitions-search-phrase");
        if (selectedText != "" && selectedText == curPhrase) {
            menuItem1.hidden = true;
        }
        

    } else {
        menuItem2.hidden = menuItem3.hidden = true;
    }
}

this.onMenuDefinitions = function (popup) { 

    var htmlDoc = document.popupNode.ownerDocument;
    var htmlWin = htmlDoc.defaultView;

    var selection = InlineDefinitionsExtension.getSelection();
    var selectedText = selection.join(" ");   
    var url = "http://www.google.com/search?q=" + selection.join("+").toLowerCase() + "&tbs=dfn:1";
    InlineDefinitionsExtension.log("onInlineDefinitions() : url = " + url);

    var Application = Components.classes["@mozilla.org/fuel/application;1"].getService(Components.interfaces.fuelIApplication);
    var currentlyOpenWindow = Application.storage.get("inlinedefinitions.currentlyOpenWindow", null); // shared storage between multiple browser windows
    InlineDefinitionsExtension.log("currentlyOpenWindow = " + currentlyOpenWindow);

    /* find existing windows */
    var definitionsWindow = null;
    var domWindow = null;
    if (currentlyOpenWindow != null) {
        var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
        var windowEnumerator = windowMediator.getEnumerator(null);
        while(windowEnumerator.hasMoreElements()) {  
            var win = windowEnumerator.getNext();  
            InlineDefinitionsExtension.log("existing window: " + win.content);
            if (win.content == currentlyOpenWindow) {
                definitionsWindow = win;
                domWindow = win.content;
                break;
            }
        }
    }

    /* load definitions url */
    if (definitionsWindow != null && domWindow != null) {
        InlineDefinitionsExtension.log("found existing window");
        definitionsWindow.content.document.location.href = url;

    } else {
        var strTop = "top=" + htmlWin.outerHeight / 2;
        var strOuterHeight = "outerHeight=" + htmlWin.outerHeight / 2;
        var strFeatures = "toolbar,location,resizable,scrollbars,alwaysRaised";
        domWindow = window.open(url, "inlinedefinitions_window", strFeatures + "," +  strTop + "," + strOuterHeight);
        if (domWindow == null) {
            alert("Unable to open a new window for definitions..");
        }
        InlineDefinitionsExtension.log("created new window: " + domWindow);

        Application.storage.set("inlinedefinitions.currentlyOpenWindow", domWindow);

        /* add event handler for when the definitions are loaded */
        var chromeWindow = domWindow
                .QueryInterface(Components.interfaces.nsIInterfaceRequestor).getInterface(Components.interfaces.nsIWebNavigation)
                .QueryInterface(Components.interfaces.nsIDocShellTreeItem).rootTreeItem
                .QueryInterface(Components.interfaces.nsIInterfaceRequestor).getInterface(Components.interfaces.nsIDOMWindow);
                // (based on https://developer.mozilla.org/en/Code_snippets/Tabbed_browser)

        InlineDefinitionsExtension.log("created new window: " + chromeWindow);
        chromeWindow.gBrowser.addEventListener("load", this.onDefinitionsLoaded, true);
    }

}

this.onDefinitionsLoaded = function(event) {
    var doc = event.originalTarget;
    var href = doc.location.href;
    if (href == "about:blank") {
        return;
    }
    InlineDefinitionsExtension.log("window loaded: " + href);

    // what a hack
    var anchor = doc.createElement("a"); anchor.setAttribute('name', 'webdefinitions');
    var scrollDiv = doc.querySelector("div#center_col");
    scrollDiv.parentNode.insertBefore(anchor, scrollDiv);

    var scrollTo = InlineDefinitionsExtension.findPos(scrollDiv);
    InlineDefinitionsExtension.log("scrolling to " + scrollTo);

    // this kinda worked, but is even more hacky
    doc.defaultView.setTimeout( function() { doc.defaultView.scroll(0, scrollTo); }, 50);

    // this doesn't work because Google's onload change the scroll position...
    // can't figure out how to run after them (yet..)
    //doc.addEventListener("load", function(event) { frame.contentWindow.scroll(0, scrollTo); }, true);

    scrollDiv.parentNode.removeChild(anchor);

    doc.defaultView.focus();

}
    
this.onMenuClose = function (popup) { 
}

this.onMenuBack = function (popup) { 
}

this.isInsideDefinitionsWindow = function(node) {
    return false;
}

this.getSelection = function() {
    
    var node = document.popupNode;
    var selection = "";

    if ((node instanceof HTMLTextAreaElement) || (node instanceof HTMLInputElement && node.type == "text")) {
        selection = node.value.substring(node.selectionStart, node.selectionEnd);
    } 
    else {
        var focusedWindow = new XPCNativeWrapper(document.commandDispatcher.focusedWindow, 'document', 'getSelection()');
        selection = focusedWindow.getSelection().toString();
    }
    //node.style.outline = "2px dashed blue !important"

    // Limit length to 150 to optimize performance. Longer does not make sense
    if (selection.length>=150){
        selection = selection.substring(0, 149);
    }
    // Strip newlines
    selection = selection.replace(/(\n|\r|\t)+/g, " ");
    // Strip spaces at start and end.
    selection = selection.replace(/(^\s+)|(\s+$)/g, "");

    selection = selection.split(" ");
   
    // Remove certain characters at the beginning and end of every word
    for (i=0; i<selection.length; i++){
        selection[i]=selection[i].replace(/^(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+/, "");
        selection[i]=selection[i].replace(/(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+$/, "");
    }   

    return selection; 
}

//Finds y value of given object
this.findPos = function (obj) {
    var pos = 0;
    if (obj.offsetParent) {
        do {
            pos += obj.offsetTop;
        } while (obj = obj.offsetParent);
        return pos;
	}
}

this.close = function () { 
    InlineDefinitionsExtension.log("close");
}

}).apply(InlineDefinitionsExtension);


window.addEventListener("load", InlineDefinitionsExtension.init,false);

/* vi: set et ai ts=4 sw=4: */
